% startup.m:   execute when MATLAB starts up.
%                1) Run 'OMpath' to add OMtools directories to path
%                2) Move to OMtools main directory

% Written by:  Jonathan Jacobs
%              December 1995 - February 2011  (last mod: 02/09/11)

% 21 Mar 2003: added '/' as sep character for MATLAB 6.5 (OS X, yay!)
% 9 Feb 2011: completely redone.  now relies on 'findomprefs' and code from 'findomtools'
%   which give greater flexibility in where OMtools and omprefs are located.

function startup( null )

global dataroot OMroot

[comp, maxsize] = computer;
vers=version;
P = path;
if comp(1) == 'M' & vers(1)<='5' % 'classic' Mac OS
   sep  = ':';                   % between dirs in a single path
   sep2 = ';';                   % between path entries
 elseif comp(1) == 'P'           % PC
   sep  = '\'; 
   sep2 = ';';
 else                            % Unix (including Mac OS X)
   sep  = '/';
   sep2 = ':';    
end

% Does 'omtools' folder exist? It can exist in a number of places:
% original location: matlabroot.  No longer a good idea, but will allow it.
% other locations: inside folder containing MATLAB; inside toolbox

oldpath = pwd;
comp = lower( computer('arch') );
if strcmp(comp(1),'m') | strcmp(comp(1),'g')
	homedir = getenv('HOME');
	documents = 'documents';
 elseif strcmp(comp(1),'p')| strcmp(comp(1),'w')
   homedir = getenv('USERPROFILE');
   documents = 'My Documents';
end 

cd(matlabroot)
cd ..
matlabdir = pwd;
omtoolspath = '';

locations = {
				  {'matlabdir'}; ...
				  {'matlabroot'}; ...
			     {'matlabroot', 'toolbox'}; ...
			     {'homedir', 'documents', 'MATLAB'}; ...
			   };

omtf=0;								%% omtools folders found
for j=1:length(locations)
	dir_err=0;
	
   temp=eval(char( locations{j}(1) ));
   if ~exist( temp, 'dir')    
       disp( ['dir_error: ' locations{j}] );
       continue;
   end
   cd(temp)
	
	% if there are subdirectories, navigate to them.
	numsubdir = length(locations{j});
	if numsubdir > 1
		dir_err=0;
		for k=2:numsubdir
			temp=char( locations{j}(k) );
			eval('cd(temp)','dir_err=1;')
		end % for k
	end %if numsubdir
	
	if ~dir_err
		dirfiles = dir;
		for i = 1:length(dirfiles)
			  temp = deblank(lower(dirfiles(i).name));
			if strcmp( temp, 'omtools')
				omtf = omtf+1;
				omtoolspath{omtf} = [pwd sep 'OMtools'];
			end
		end %for i
	end %if ~dir_err	

end %for j

if omtf < 1
	disp('Could not find the OMtools folder. Make sure it is installed in an appropriate location.')
	disp('Allowed directories are: matlabroot; the folder containing matlabroot; ')
	disp('the MATLAB toolbox folder (against Mathworks rules!!!); ')
	disp('inside your personal MATLAB directory in your home directory.')
	omtoolspath = '';
	return
 elseif omtf == 1
 	omtoolspath = char(omtoolspath{1});
	cd(omtoolspath)
	cd ..
	OMroot = pwd;		%%OMroot is the folder _containing_ OMtools
 elseif omtf > 1
   disp('Multiple OMtools folders found:')
   for m=1:omtf
   	disp( char(omtoolspath{m}) )
   end
   disp('You should remove or rename all but one of the OMtools folders listed above.')
  	omtoolspath = '';
	return
end	

% find 'OMpath' in 'datadirs'
dirErr=0;
eval( ['cd ' '''' OMroot  sep 'OMtools' sep 'datadirs' '''' ;],['dirErr=1;'] )
eval( ['ompath'] )

% look for DPI preferences in omprefs and set
cd(findomprefs)
if exist('DPIprefs.txt','file')
   ppi = load('DPIprefs.txt');
   set(0,'ScreenPixelsPerInch',ppi)
 else   
   dpi('set')  
end

%eval( ['ergpath'] )
%colordef none