% countobj.m:  Count the user-specified objects in axes.
% If no object type is specified, 'lines' will be counted.
% usage:   [num, hlist] = countobj(type, ax)   
%   where type is 'line', 'patch', 'text' or 'surf' and ax is the 
%   desired axis.  If none is specified, the current one will be used.
%   num is the number of objects, hlist (optional) is a list of handles

% Written by:  Jonathan Jacobs
%              March 1998 - October 2002  (last mod: 10/22/02)

function [num,hlist] = countobj(objtype, ax)

% default
if nargin == 0, ax = gca; objtype = 'line'; end;
if nargin < 2,  ax = gca; end

gf = get(ax,'Parent');

% check validity of handle given
if isempty(find(get(gf,'Children') == ax)),
    disp('countobj: Invalid axis handle!');
    num = -1;
    return
end

num = 0; hlist = [];

% walk along the list of children, picking only the appropriate type
c = get(ax,'Children');
for i = 1:length(c),
    if strcmp(get(c(i), 'Type'), objtype)
       hlist = [hlist ; c(i)];
    end
end;

num = length(hlist);
