% loadfig.m:  Load a figure saved by 'savefig'.
% You may call 'loadfig' with a file name, INCLUDING THE FULL PATH
% e.g.: loadfig('EyeMac:figures:testfig.m')
% If no file name is given, 'loadfig' presents a file select box.
% The load path is stored in 'loadfig.ini' in the 'omprefs' directory.
% (If one does not exist, it will be created in your MATLAB root directory)

% Written by Jonathan Jacobs
%             March 1997 - September 2004  (last mod: 09/23/04)

function figHandle = loadfig( filename )

[comp, maxsize] = computer;
v=version;
if comp(1) == 'M'
   if v(1)<= '5'
      sep = ':';
    else
      sep = '/';
   end
 else
   sep = '\';
end

if v(1) <= '5'
   fileselect = '*.m';
 else
   fileselect = {'*.m; *.fig'};
end

currentdir = pwd;

gp_err = 0;
cd(matlabroot)
eval('cd(findomprefs)','gp_err=1;')
if gp_err % must make a omprefs directory and create a 'loadfig.ini' file
   mkdir('omprefs')
   cd('omprefs')
   save('loadfig.ini', [matlabroot, sep]);
end


if nargin < 1

   loaderr = 0;
   loadPath = pwd;
   % load the last path, stored in 'loadfig.ini' as 'loadPath'
   eval(['load loadfig.ini -mat'],['loaderr=1;']);
   if loaderr, loadPath = pwd; end 

   %if the last char in the path is a separator, remove it.
   if strcmp(loadPath(end),sep)
      loadPath = loadPath(1:end-1);
   end
   if exist(loadPath,'dir')
      cd(loadPath)
    else
      disp(['The directory ' '''' loadPath '''' ' is no longer present.'])
      disp('Choose another directory from which to load the figure.')
      cd(currentdir)
   end

  [filename,loadPath] = uigetfile(fileselect, 'Load a saved Figure (.m or .fig)');
  if filename == 0
     disp( 'Canceled.')
     cd(currentdir)
     return
  end
  if strcmp(loadPath(end),sep)
     loadPath = loadPath(1:end-1);
  end
  cd(currentdir)

  % update 'loadPath' in 'loadfig.ini'
  %cd(matlabroot)
  cd(findomprefs)
  if (loadPath)
     eval('save loadfig.ini loadPath ');
  end

else
   % extract filename and loadPath from user-input 'filename' argument
   sepPos = findstr(filename,sep);
   loadPath = filename(1:sepPos(end)-1);
   filename = filename(sepPos(end)+1:end);
end


% time to load the file
if filename
   eval(['cd ' '''' loadPath ''''])
   disp(['loading: ' '''' loadPath sep filename ''''])
   if v(1) <= '5'
      pointPos = findstr(filename,'.');
      eval([filename(1:pointPos-1)])
    else
      open(filename)
   end
   set(gcf,'Name', filename);
 else
   figHandle = -1;
   error('Figure file doesn''t exist!')
   return
end

%%%% Hopefully this is only temp
%%%% to fix the MATLAB 'renderer'/'renderermode' bug
set(gcf,'renderermode','auto');

cd(currentdir)
return
