% hv2r.m: create a vector 'r' that is the pythagorean combination of
% the horizontal and vertical components of the eye movement.
% Magnitude AND direction are properly preserved.
% Usage: r = hv2r(hor, vrt, disp);  where a value of 1 for 'disp' will create
%                                   a figure with the hor, vrt and r data.

% Written by:  Jonathan Jacobs
%              July 2002  (last mod: 07/26/02)

function r = hv2r(hor, vrt, disp)

if nargin==0, help hv2r; return; end

if nargin<3, disp=0; end

[hr,hc] = size(hor);
[vr,vc] = size(vrt);

% make sure data is ordered in a column
if hr<hc, hor=hor'; end
if vr<vc, vrt=vrt'; end

hor_sqr = hor .* abs(hor);
vrt_sqr = vrt .* abs(vrt);

horvrt_sqr = hor_sqr + vrt_sqr;
horvrt_sgn = sign(horvrt_sqr);
r = sqrt(abs(horvrt_sqr)) .* horvrt_sgn;


if (disp)
   figure
   t=maket(hor);
   subplot(3,1,1)
   plot(t, hor)
   ylabel('Hor. data')
   subplot(3,1,2)
   plot(t, vrt)
   ylabel('Vrt. data')
   subplot(3,1,3)
   plot(t, r)
   ylabel('R vector')
   xlabel('Time (seconds)')
end
