% efitfun.m: exponential fit function used by efit.m
%	EFITFUN(lambda) returns the error between the data and the
%	values computed by the current function of lambda.
%	EFITFUN assumes a function of the form
%
%	  y =  c(1)*exp(-lambda(1)*t) + ... + c(n)*exp(-lambda(n)*t)
%
%	with n linear parameters and n nonlinear parameters.

%	Copyright (c) 1984-94 by The MathWorks, Inc.

function [err] = efitfun(lambda)

global xdata ydata c plotH

t = xdata';
y = ydata';
A = zeros(length(t),length(lambda));
for j = 1:length(lambda)
   A(:,j) = exp(-lambda(j)*t);
end
c = A\y;
ynew = A*c;
set(plotH,'ydata',ynew)
%drawnow
err = norm(ynew-y);
