% posify.m: Takes an input vector and returns a vector with zeros wherever
% the original values were negative or zero.
%
% USAGE: [outarr, outpts] = posify(inarr);
% where inarr is the original vector      (e.g. lh)
%       outarr is the resulting vector    (e.g. lht)
%   and outpts is the list of points that were > 0.

% Written by:  Jonathan Jacobs
%              January 2000  (last mod: 01/04/00)

function [outarr,outpts] = posify(inarr)

[r,c]=size(inarr);
if (r>1) & (c>1)
   disp('Please input a ONE-dimensional array.')
   return
end

outpts = find(inarr>0);
outarr = NaN*ones(r,c);
outarr(outpts) = inarr(outpts);
[numoutpts, numcols] = size(outpts);

disp(['There were ' num2str(numoutpts) ' positive values.'])
