% time.m: show the date and time
% Usage: datetimestring = time(date_or_time, showday, hourformat, showseconds);
%
% where date_or_time = 'd' -- date only
%                      't' -- time only
%                      'b' -- both date and time    (default)
%       showday      =  0  -- don't show day of the week 
%                       1  -- show day of the week  (default)
%       hourformat   = 12  -- 12 hour format        (default)
%                      24  -- 24 hour format
%       showseconds  =  0  -- don't show seconds 
%                       1  -- show seconds          (default)
%
% When called without an output argument, result is displayed on screen.
% When called with an output argument, display is suppressed.

% Written by:  Jonathan Jacobs
%              August 1998 - May 2001 (last mod: 05/15/01)

function out=time(d_or_t, showdow, hourformat, showsec)

if nargin < 4
   d_or_t = 'b';
   showdow = 1;
   hourformat = 12;
   showsec = 1;
end

a=date;
b=clock;

year=b(1);
month=b(2);
dt=b(3);

if(0)
   year = 1964;
   month = 12;
   dt = 5;
end

hour24=b(4);
hour12=b(4);
minutes=b(5);
seconds=fix(b(6));

% find the day of the week
dowlist = ['Sat';'Sun';'Mon';'Tue';'Wed';'Thu';'Fri';];
startyear = 1904;

% how many leap days since startyear?
numyears = year-startyear;
numly = fix(numyears/4);
numly = rem(numly,7)+7;   %convert negative numbers...
numly = rem(numly,7);

% ignore century years not divisible by 400...
leapcent = 0;
if year/100 == fix(year/100)
   if year/400 == fix(year/400)
      leapcent = 1;
   end 
end

if (year/4 ~= fix(year/4)) | ~leapcent
   isleap = 0;
   monthoffset = [0 3 3 6 1 4 6 2 5 0 3 5];
 else
   isleap = 1;
   monthoffset = [0 3 4 0 2 5 0 3 6 1 4 6];
end

% '-1' is balanced by '+1' since ML doesn't like zero as a matrix index.
dowoffset = numyears + monthoffset(month) + dt + numly - 1;
dowoffset = rem(dowoffset,7)+7;   %convert negative numbers...
dowoffset = rem(dowoffset,7)+1;
dowoffset = abs(dowoffset);
dow = dowlist(dowoffset,:);
%keyboard

if hour24>12
   hour12=hour24-12;
end

ampm=' AM';
if hour24>=12
   ampm=' PM';
end

monthlist=['Jan';'Feb';'Mar';'Apr';'May';'Jun';...
           'Jul';'Aug';'Sep';'Oct';'Nov';'Dec'];

if showdow
   datestr = [dow ', ' monthlist(month,:) ' ' num2str(dt) ', ' num2str(year)];
 else
   datestr = [monthlist(month,:) ' ' num2str(dt) ', ' num2str(year)];
end

if minutes < 10
   minstr = ['0' num2str(minutes)];
 else
   minstr = num2str(minutes);
end

if hour24 < 10
   hour24str = ['0' num2str(hour24)];
 else
   hour24str = num2str(hour24);
end

if showsec
   if seconds < 10
      secstr = ['0' num2str(seconds)];
    else
      secstr = num2str(seconds);
   end
   time24str = [hour24str ':' minstr ':' secstr];
   time12str = [num2str(hour12) ':' minstr ':' secstr ampm];
 else
   time24str = [hour24str ':' minstr];
   time12str = [num2str(hour12) ':' minstr ampm];
end

if hourformat == 24
   timestr = time24str;
 else
   timestr = time12str;
end

if lower(d_or_t) == 'd'
   dtstr = datestr;
 elseif lower(d_or_t) == 't'
   dtstr = timestr;
 elseif lower(d_or_t) == 'b'
   dtstr = [datestr '. ' timestr];
end

%if(1)
if nargout == 0
   disp(dtstr)
   return
end

out = dtstr;
return
