function [h] = crsrmove(ax,name,pos)
%CRSRMOVE Move axes cursor.
%       H=CRSRMOVE(AXES,'NAME',POSITION) moves the named cursor
%       to a new location. It does not change the visibility of
%       the cursor. If the axes handle is not provided, the
%       current axis is used.
%
%       See also CRSRCR, CRSRDEL, CRSRON, CRSROFF

%       Dennis W. Brown 1-10-94
%       Copyright (c) 1994 by Dennis W. Brown
%       May be freely distributed.
%       Not for use in commercial products.
%
%   modified by Jonathan Jacobs to allow dragging of the cursor
%   into sections of data that are NaN.  (Sept 1997)

% Modifications by Jonathan Jacobs
%                  September 1997 - July 1999 (last mod: 07/06/99)

% handles to readouts
temp = get(gca,'UserData');     
editH = temp{1};

% get handle to cursor
h = findline(ax,name);

% get current axis data
x = get(ax,'XLim');
y = get(ax,'YLim');

% get current cursor location
xx = get(h,'XData');
yy = get(h,'YData');

% also added this to most of the routines that call
% "crsrmove", for "belt and suspenders" safety.
if isnan(pos)
   pos = (y(1)+y(2))/2;
end

% move the cursor
if xx(1) == xx(2),          % vertical cursor

    % actually move the damn thing
    set(h,'XData',[pos pos]);
    % update the numeric display
    if( name < 1999 ),
    	set(editH(9),'String',num2str(pos));
    else
    	set(editH(13),'String',num2str(pos));
    end
    % update the "Delta X" field
    diff = str2num(get(editH(9),'String')) ... 
          - str2num(get(editH(13),'String'));
    set(editH(11),'String',num2str(abs(diff)))

elseif yy(1) == yy(2) | any(isnan(yy)),      % horizontal cursor

    % actually move the damn thing
    set(h,'YData',[pos pos]);
    % update the numeric display
    if( name < 1999 ),
    	set(editH(10),'String',num2str(pos));
    else
    	set(editH(14),'String',num2str(pos));
    end
    % update the "Delta Y" field
    diff = str2num(get(editH(10),'String')) ... 
          - str2num(get(editH(14),'String'));
    set(editH(12),'String',num2str(abs(diff)))

else
%    disp('cursormv: Invalid cursor found...');
%    keyboard
    error('cursormv: Invalid cursor found...');
end;
